@@ -25,6 +25,7 @@ import numpy
 from tqdm.auto import tqdm
 
 from deepsparse.benchmark import BenchmarkResults
+from deepsparse.utils import model_to_path, override_onnx_input_shapes
 
 
 try:
@@ -50,12 +51,17 @@ except ImportError:
     )
 
 
-__all__ = ["Engine", "compile_model", "benchmark_model", "analyze_model", "Scheduler"]
+__all__ = [
+    "Engine",
+    "compile_model",
+    "benchmark_model",
+    "analyze_model",
+    "Scheduler",
+]
 
 
 ARCH = cpu_architecture()
-CORES_PER_SOCKET = ARCH.available_cores_per_socket
-NUM_SOCKETS = ARCH.available_sockets
+NUM_CORES = ARCH.num_physical_cores
 AVX_TYPE = ARCH.isa
 VNNI = ARCH.vnni
 
@@ -72,50 +78,28 @@ class Scheduler(Enum):
     - default: maps to single_stream
     - single_stream: requests from separate threads execute serially
     - multi_stream: requests from separate threads execute in parallel
+    - elastic: requests from separate threads are distributed across NUMA nodes
     """
 
     default = "default"
     single_stream = "single_stream"
     multi_stream = "multi_stream"
+    elastic = "elastic"
 
     @staticmethod
     def from_str(key: str):
         if key in ("default"):
             return Scheduler.default
-        elif key in ("single", "single_stream"):
+        elif key in ("sync", "single", "single_stream"):
             return Scheduler.single_stream
-        elif key in ("multi", "multi_stream"):
+        elif key in ("async", "multi", "multi_stream"):
             return Scheduler.multi_stream
+        elif key in ("elastic"):
+            return Scheduler.elastic
         else:
             raise ValueError(f"unsupported Scheduler: {key}")
 
 
-def _model_to_path(model: Union[str, Model, File]) -> str:
-    if not model:
-        raise ValueError("model must be a path, sparsezoo.Model, or sparsezoo.File")
-
-    if isinstance(model, str) and model.startswith("zoo:"):
-        # load SparseZoo Model from stub
-        if sparsezoo_import_error is not None:
-            raise sparsezoo_import_error
-        model = Zoo.load_model_from_stub(model)
-
-    if Model is not object and isinstance(model, Model):
-        # default to the main onnx file for the model
-        model = model.onnx_file.downloaded_path()
-    elif File is not object and isinstance(model, File):
-        # get the downloaded_path -- will auto download if not on local system
-        model = model.downloaded_path()
-
-    if not isinstance(model, str):
-        raise ValueError("unsupported type for model: {}".format(type(model)))
-
-    if not os.path.exists(model):
-        raise ValueError("model path must exist: given {}".format(model))
-
-    return model
-
-
 def _validate_batch_size(batch_size: int) -> int:
     if batch_size < 1:
         raise ValueError("batch_size must be greater than 0")
@@ -125,7 +109,7 @@ def _validate_batch_size(batch_size: int) -> int:
 
 def _validate_num_cores(num_cores: Union[None, int]) -> int:
     if not num_cores:
-        num_cores = CORES_PER_SOCKET
+        num_cores = NUM_CORES
 
     if num_cores < 1:
         raise ValueError("num_cores must be greater than 0")
@@ -133,16 +117,6 @@ def _validate_num_cores(num_cores: Union[None, int]) -> int:
     return num_cores
 
 
-def _validate_num_sockets(num_sockets: Union[None, int]) -> int:
-    if not num_sockets:
-        num_sockets = NUM_SOCKETS
-
-    if num_sockets < 1:
-        raise ValueError("num_sockets must be greater than 0")
-
-    return num_sockets
-
-
 def _validate_scheduler(scheduler: Union[None, str, Scheduler]) -> Scheduler:
     if not scheduler:
         scheduler = Scheduler.default
@@ -174,13 +148,11 @@ class Engine(object):
         prefixed by 'zoo:', a SparseZoo Model object, or a SparseZoo ONNX File
         object that defines the neural network
     :param batch_size: The batch size of the inputs to be used with the engine
-    :param num_cores: The number of physical cores to run the model on.
-        Pass None or 0 to run on the max number of cores
-        in one socket for the current machine, default None
-    :param num_sockets: The number of physical sockets to run the model on.
-        Pass None or 0 to run on the max number of sockets for the
-        current machine, default None
+    :param num_cores: The number of physical cores to run the model on. If more
+        cores are requested than are available on a single socket, the engine
+        will try to distribute them evenly across as few sockets as possible.
     :param scheduler: The kind of scheduler to execute with. Pass None for the default.
+    :param input_shapes: The list of shapes to set the inputs to. Pass None to use model as-is.
     """
 
     def __init__(
@@ -188,23 +160,33 @@ class Engine(object):
         model: Union[str, Model, File],
         batch_size: int,
         num_cores: int,
-        num_sockets: int = None,
         scheduler: Scheduler = None,
+        input_shapes: List[List[int]] = None,
     ):
-        self._model_path = _model_to_path(model)
+        self._model_path = model_to_path(model)
         self._batch_size = _validate_batch_size(batch_size)
         self._num_cores = _validate_num_cores(num_cores)
-        self._num_sockets = _validate_num_sockets(num_sockets)
         self._scheduler = _validate_scheduler(scheduler)
         self._cpu_avx_type = AVX_TYPE
         self._cpu_vnni = VNNI
-        self._eng_net = LIB.deepsparse_engine(
-            self._model_path,
-            self._batch_size,
-            self._num_cores,
-            self._num_sockets,
-            self._scheduler.value,
-        )
+
+        if input_shapes:
+            with override_onnx_input_shapes(
+                self._model_path, input_shapes
+            ) as model_path:
+                self._eng_net = LIB.deepsparse_engine(
+                    model_path,
+                    self._batch_size,
+                    self._num_cores,
+                    self._scheduler.value,
+                )
+        else:
+            self._eng_net = LIB.deepsparse_engine(
+                self._model_path,
+                self._batch_size,
+                self._num_cores,
+                self._scheduler.value,
+            )
 
     def __call__(
         self,
@@ -270,11 +252,11 @@ class Engine(object):
         return self._num_cores
 
     @property
-    def num_sockets(self) -> int:
+    def scheduler(self) -> Scheduler:
         """
-        :return: The number of sockets the engine is compiled to run on
+        :return: The kind of scheduler to execute with
         """
-        return self._num_sockets
+        return self._scheduler
 
     @property
     def cpu_avx_type(self) -> str:
@@ -533,7 +515,6 @@ class Engine(object):
             "onnx_file_path": self._model_path,
             "batch_size": self._batch_size,
             "num_cores": self._num_cores,
-            "num_sockets": self._num_sockets,
             "scheduler": self._scheduler,
             "cpu_avx_type": self._cpu_avx_type,
             "cpu_vnni": self._cpu_vnni,
@@ -544,34 +525,32 @@ def compile_model(
     model: Union[str, Model, File],
     batch_size: int = 1,
     num_cores: int = None,
-    num_sockets: int = None,
     scheduler: Scheduler = None,
+    input_shapes: List[List[int]] = None,
 ) -> Engine:
     """
     Convenience function to compile a model in the DeepSparse Engine
     from an ONNX file for inference.
-    Gives defaults of batch_size == 1, num_cores == None and num_sockets = None
-    (will use all physical cores available on all available sockets).
+    Gives defaults of batch_size == 1, num_cores == None
+    (will use all physical cores available on the system).
 
     :param model: Either a path to the model's onnx file, a SparseZoo model stub
         prefixed by 'zoo:', a SparseZoo Model object, or a SparseZoo ONNX File
         object that defines the neural network
     :param batch_size: The batch size of the inputs to be used with the model
     :param num_cores: The number of physical cores to run the model on.
-        Pass None or 0 to run on the max number of cores
-        in one socket for the current machine, default None
-    :param num_sockets: The number of physical sockets to run the model on.
-        Pass None or 0 to run on the max number of sockets for the
-        current machine, default None
+        Pass None or 0 to run on the max number of cores for the current
+        machine; default None
     :param scheduler: The kind of scheduler to execute with. Pass None for the default.
+    :param input_shapes: The list of shapes to set the inputs to. Pass None to use model as-is.
     :return: The created Engine after compiling the model
     """
     return Engine(
-        model,
-        batch_size,
-        num_cores,
-        num_sockets=num_sockets,
+        model=model,
+        batch_size=batch_size,
+        num_cores=num_cores,
         scheduler=scheduler,
+        input_shapes=input_shapes,
     )
 
 
@@ -585,7 +564,6 @@ def benchmark_model(
     include_inputs: bool = False,
     include_outputs: bool = False,
     show_progress: bool = False,
-    num_sockets: int = None,
     scheduler: Scheduler = None,
 ) -> BenchmarkResults:
     """
@@ -602,7 +580,7 @@ def benchmark_model(
     :param batch_size: The batch size of the inputs to be used with the model
     :param num_cores: The number of physical cores to run the model on.
         Pass None or 0 to run on the max number of cores
-        in one socket for the current machine, default None
+        for the current machine; default None
     :param inp: The list of inputs to pass to the engine for benchmarking.
         The expected order is the inputs order as defined in the ONNX graph.
     :param num_iterations: The number of iterations to run benchmarking for.
@@ -616,17 +594,13 @@ def benchmark_model(
     :param include_outputs: If True, outputs from forward passes during benchmarking
         will be added to the results. Default is False
     :param show_progress: If True, will display a progress bar. Default is False
-    :param num_sockets: The number of physical sockets to run the model on.
-        Pass None or 0 to run on the max number of sockets for the
-        current machine, default None
     :param scheduler: The kind of scheduler to execute with. Pass None for the default.
     :return: the results of benchmarking
     """
     model = compile_model(
-        model,
-        batch_size,
-        num_cores,
-        num_sockets=num_sockets,
+        model=model,
+        batch_size=batch_size,
+        num_cores=num_cores,
         scheduler=scheduler,
     )
 
@@ -650,7 +624,6 @@ def analyze_model(
     optimization_level: int = 1,
     imposed_as: Optional[float] = None,
     imposed_ks: Optional[float] = None,
-    num_sockets: int = None,
     scheduler: Scheduler = None,
 ) -> dict:
     """
@@ -669,7 +642,7 @@ def analyze_model(
     :param batch_size: The batch size of the inputs to be used with the model
     :param num_cores: The number of physical cores to run the model on.
         Pass None or 0 to run on the max number of cores
-        in one socket for the current machine, default None
+        for the current machine; default None
     :param num_iterations: The number of times to repeat execution of the model
         while analyzing, default is 20
     :param num_warmup_iterations: The number of times to repeat execution of the model
@@ -684,20 +657,14 @@ def analyze_model(
         Will force all prunable layers in the graph to have weights with
         this desired sparsity level (percentage of 0's in the tensor).
         Beneficial for seeing how pruning affects the performance of the model.
-    :param num_sockets: The number of physical sockets to run the model on.
-        Pass None or 0 to run on the max number of sockets for the
-        current machine, default None
     :param scheduler: The kind of scheduler to execute with. Pass None for the default.
     :return: the analysis structure containing the performance details of each layer
     """
-    model = _model_to_path(model)
+    model = model_to_path(model)
     num_cores = _validate_num_cores(num_cores)
     batch_size = _validate_batch_size(batch_size)
-    num_sockets = _validate_num_sockets(num_sockets)
     scheduler = _validate_scheduler(scheduler)
-    eng_net = LIB.deepsparse_engine(
-        model, batch_size, num_cores, num_sockets, scheduler.value
-    )
+    eng_net = LIB.deepsparse_engine(model, batch_size, num_cores, scheduler.value)
 
     return eng_net.benchmark(
         inp,
